"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Db = void 0;
const pg_listen_1 = __importDefault(require("pg-listen"));
const ts_log_1 = require("ts-log");
const MODULE_NAME = 'Db';
class Db {
    constructor(pgClientConfig, logger = ts_log_1.dummyLogger) {
        this.logger = logger;
        this.state = null;
        this.pgSubscriber = pg_listen_1.default(pgClientConfig, {
            parse: (value) => value
        });
    }
    init({ onDbInit, onDbSetup }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== null)
                return;
            this.state = 'initializing';
            this.pgSubscriber.events.on('connected', () => __awaiter(this, void 0, void 0, function* () {
                this.logger.debug({ module: MODULE_NAME }, 'pgSubscriber: Connected');
                yield onDbSetup();
            }));
            this.pgSubscriber.events.on('reconnect', (attempt) => {
                this.logger.warn({ module: MODULE_NAME }, `pgSubscriber: Reconnecting attempt ${attempt}`);
            });
            this.pgSubscriber.events.on('error', (error) => {
                this.logger.error({ module: MODULE_NAME, err: error }, 'pgSubscriber');
                process.exit(1);
            });
            this.pgSubscriber.notifications.on('cardano_db_sync_startup', (payload) => __awaiter(this, void 0, void 0, function* () {
                switch (payload) {
                    case 'init':
                        this.logger.warn({ module: 'Db' }, 'pgSubscriber: cardano-db-sync-extended starting, schema will be reset');
                        yield onDbInit();
                        break;
                    case 'db-setup':
                        yield onDbSetup();
                        break;
                    default:
                        this.logger.error({ module: MODULE_NAME }, `DbClient.pgSubscriber: Unknown message payload ${payload}`);
                }
            }));
            try {
                yield this.pgSubscriber.connect();
                yield this.pgSubscriber.listenTo('cardano_db_sync_startup');
                this.state = 'initialized';
            }
            catch (error) {
                this.logger.error({ err: error });
            }
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'initialized')
                return;
            yield this.pgSubscriber.close();
        });
    }
}
exports.Db = Db;
//# sourceMappingURL=Db.js.map