import { Schema } from '@cardano-ogmios/client';
import { DataFetcher } from '@cardano-graphql/util';
import { GraphQLSchema } from 'graphql';
import { AdaPots, Asset, AssetSupply, Block, PaymentAddressSummary, ShelleyProtocolParams } from './graphql_types';
import { Logger } from 'ts-log';
import { AssetMetadataAndHash, AssetMetadataHashAndId, AssetWithoutTokens } from './typeAliases';
export declare type AdaPotsToCalculateSupply = {
    circulating: AssetSupply['circulating'];
    reserves: AdaPots['reserves'];
};
export declare class HasuraClient {
    readonly hasuraCliPath: string;
    readonly hasuraUri: string;
    readonly lastConfiguredMajorVersion: number;
    private logger;
    private client;
    private applyingSchemaAndMetadata;
    adaPotsToCalculateSupplyFetcher: DataFetcher<AdaPotsToCalculateSupply>;
    private state;
    schema: GraphQLSchema;
    constructor(hasuraCliPath: string, hasuraUri: string, pollingInterval: number, lastConfiguredMajorVersion: number, logger?: Logger);
    private getAdaPotsToCalculateSupply;
    private hasuraCli;
    initialize(): Promise<void>;
    shutdown(): Promise<void>;
    applySchemaAndMetadata(): Promise<void>;
    buildHasuraSchema(): Promise<GraphQLSchema>;
    deleteAssetsAfterSlot(slotNo: Block['slotNo']): Promise<number>;
    getCurrentProtocolVersion(): Promise<ShelleyProtocolParams['protocolVersion']>;
    getMostRecentPointWithNewAsset(): Promise<Schema.Point | null>;
    getPaymentAddressSummary(address: string, atBlock?: number): Promise<PaymentAddressSummary>;
    getMeta(nodeTipSlotNumber: number): Promise<{
        initialized: boolean;
        syncPercentage: number;
    }>;
    hasAsset(assetId: Asset['assetId']): Promise<boolean>;
    getAssetMetadataHashesById(assetIds: Asset['assetId'][]): Promise<AssetMetadataHashAndId[]>;
    addAssetMetadata(asset: AssetMetadataAndHash): Promise<void>;
    insertAssets(assets: AssetWithoutTokens[]): Promise<any>;
}
//# sourceMappingURL=HasuraClient.d.ts.map