"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HasuraClient = void 0;
const child_process_1 = require("child_process");
const util_1 = __importStar(require("@cardano-graphql/util"));
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const graphql_1 = require("graphql");
const graphql_request_1 = require("graphql-request");
const wrap_1 = require("@graphql-tools/wrap");
const p_retry_1 = __importDefault(require("p-retry"));
const path_1 = __importDefault(require("path"));
const ts_log_1 = require("ts-log");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const epochInformationNotYetAvailable = 'Epoch information not yet available. This is expected during the initial chain-sync.';
const withHexPrefix = (value) => `\\x${value !== undefined ? value : ''}`;
class HasuraClient {
    constructor(hasuraCliPath, hasuraUri, pollingInterval, lastConfiguredMajorVersion, logger = ts_log_1.dummyLogger) {
        this.hasuraCliPath = hasuraCliPath;
        this.hasuraUri = hasuraUri;
        this.lastConfiguredMajorVersion = lastConfiguredMajorVersion;
        this.logger = logger;
        this.state = null;
        this.applyingSchemaAndMetadata = false;
        this.adaPotsToCalculateSupplyFetcher = new util_1.DataFetcher('AdaPotsToCalculateSupply', () => {
            try {
                return this.getAdaPotsToCalculateSupply();
            }
            catch (error) {
                if (error.message !== epochInformationNotYetAvailable) {
                    console.debug(error.message);
                }
                this.logger.trace({ err: error });
            }
        }, pollingInterval, this.logger);
        this.client = new graphql_request_1.GraphQLClient(`${this.hasuraUri}/v1/graphql`, {
            headers: {
                'X-Hasura-Role': 'cardano-graphql'
            }
        });
    }
    getAdaPotsToCalculateSupply() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query {
          epochs (limit: 1, order_by: { number: desc }) {
              adaPots {
                  reserves
              }
          }
          rewards_aggregate {
              aggregate {
                  sum {
                      amount
                  }
              }
          }
          utxos_aggregate {
              aggregate {
                  sum {
                      value
                  }
              }
          }
          withdrawals_aggregate {
              aggregate {
                  sum {
                      amount
                  }
              }
          }
      }`);
            const { epochs, rewards_aggregate: rewardsAggregate, utxos_aggregate: utxosAggregate, withdrawals_aggregate: withdrawalsAggregate } = result;
            if (epochs.length === 0 || ((_a = epochs[0]) === null || _a === void 0 ? void 0 : _a.adaPots) === null) {
                this.logger.debug({ module: 'HasuraClient' }, epochInformationNotYetAvailable);
                throw new Error(epochInformationNotYetAvailable);
            }
            const rewards = new bignumber_js_1.default(rewardsAggregate.aggregate.sum.amount);
            const utxos = new bignumber_js_1.default(utxosAggregate.aggregate.sum.value);
            const withdrawals = new bignumber_js_1.default(withdrawalsAggregate.aggregate.sum.amount);
            const withdrawableRewards = rewards.minus(withdrawals);
            return {
                circulating: utxos.plus(withdrawableRewards).toString(),
                reserves: (_b = epochs[0]) === null || _b === void 0 ? void 0 : _b.adaPots.reserves
            };
        });
    }
    hasuraCli(command) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                child_process_1.exec(`${this.hasuraCliPath} --skip-update-check --project ${path_1.default.resolve(__dirname, '..', 'hasura', 'project')} --endpoint ${this.hasuraUri} ${command}`, (error, stdout) => {
                    if (error) {
                        reject(error);
                    }
                    this.logger.debug({ module: 'HasuraClient' }, stdout);
                    resolve();
                });
            });
        });
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== null)
                return;
            this.state = 'initializing';
            this.logger.info({ module: 'HasuraClient' }, 'Initializing');
            yield this.applySchemaAndMetadata();
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                this.schema = yield this.buildHasuraSchema();
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Fetching Hasura schema via introspection', this.logger)
            });
            this.logger.debug({ module: 'HasuraClient' }, 'graphql-engine setup');
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                const result = yield this.client.request(graphql_request_1.gql `query {
            epochs (limit: 1, order_by: { number: desc }) {
                number
            }
        }`);
                if (result.epochs.length === 0) {
                    this.logger.debug({ module: 'HasuraClient' }, epochInformationNotYetAvailable);
                    throw new Error(epochInformationNotYetAvailable);
                }
            }), {
                factor: 1.05,
                retries: 100,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Detecting DB sync state has reached minimum progress', this.logger)
            });
            this.logger.debug({ module: 'HasuraClient' }, 'DB sync state has reached minimum progress');
            yield this.adaPotsToCalculateSupplyFetcher.initialize();
            this.state = 'initialized';
            this.logger.info({ module: 'HasuraClient' }, 'Initialized');
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.adaPotsToCalculateSupplyFetcher.shutdown();
        });
    }
    applySchemaAndMetadata() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.applyingSchemaAndMetadata)
                return;
            this.applyingSchemaAndMetadata = true;
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield this.hasuraCli('migrate apply --down all');
                yield this.hasuraCli('migrate apply --up all');
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Applying PostgreSQL schema migrations', this.logger)
            });
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield this.hasuraCli('metadata clear');
                yield this.hasuraCli('metadata apply');
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Applying Hasura metadata', this.logger)
            });
            this.applyingSchemaAndMetadata = false;
        });
    }
    buildHasuraSchema() {
        return __awaiter(this, void 0, void 0, function* () {
            const executor = ({ document, variables }) => __awaiter(this, void 0, void 0, function* () {
                const query = graphql_1.print(document);
                try {
                    const fetchResult = yield cross_fetch_1.default(`${this.hasuraUri}/v1/graphql`, {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                            'X-Hasura-Role': 'cardano-graphql'
                        },
                        body: JSON.stringify({ query, variables })
                    });
                    return fetchResult.json();
                }
                catch (error) {
                    this.logger.error({ err: error });
                    throw error;
                }
            });
            const coreTypes = [
                'Block',
                'Cardano',
                'Epoch',
                'Block',
                'Transaction'
            ];
            const schema = wrap_1.wrapSchema({
                schema: yield wrap_1.introspectSchema(executor),
                executor
            });
            for (const t of coreTypes) {
                const gqlType = schema.getType(t);
                if (!gqlType) {
                    throw new Error(`Remote schema is missing ${t}`);
                }
            }
            return schema;
        });
    }
    deleteAssetsAfterSlot(slotNo) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug({ module: 'HasuraClient', slotNo }, 'deleting assets found in tokens after slot');
            const result = yield this.client.request(graphql_request_1.gql `mutation DeleteAssetsAfterSlot($slotNo: Int!) {
          delete_assets(
              where: {
                  firstAppearedInSlot: {
                      _gt: $slotNo
                  }
              }
          ) {
              affected_rows
          }
      }`, {
                slotNo
            });
            return result.delete_assets.affected_rows;
        });
    }
    getCurrentProtocolVersion() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query {
          epochs (limit: 1, order_by: { number: desc }) {
              protocolParams {
                  protocolVersion
              }
          }
      }`);
            return (_a = result.epochs[0]) === null || _a === void 0 ? void 0 : _a.protocolParams.protocolVersion;
        });
    }
    getMostRecentPointWithNewAsset() {
        return __awaiter(this, void 0, void 0, function* () {
            let point;
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                const result = yield this.client.request(graphql_request_1.gql `query {
            assets (
                limit: 1
                offset: 1
                order_by: { firstAppearedInBlock: { slotNo: desc }}
            ) {
                firstAppearedInBlock {
                    hash
                    slotNo
                }
            }
        }`);
                if (result.errors !== undefined) {
                    throw new Error(result.errors);
                }
                if (result.assets.length !== 0) {
                    if (result.assets[0].firstAppearedInBlock === null) {
                        throw new Error('cardano-db-sync is lagging behind the asset sync operation.');
                    }
                    const { hash, slotNo } = result.assets[0].firstAppearedInBlock;
                    point = {
                        hash: hash.substring(2),
                        slot: slotNo
                    };
                }
                else {
                    point = null;
                }
            }), {
                factor: 1.5,
                retries: 1000,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Getting the most recent point with a new asset', this.logger)
            });
            return point;
        });
    }
    getPaymentAddressSummary(address, atBlock) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query PaymentAddressSummary (
          $address: String!
          $atBlock: Int
      ){
          utxos (
              where: {
                  _and: {
                      address: { _eq: $address },
                      transaction: { block: { number: { _lte: $atBlock }}}
                  }
              }
          ) {
              value
              tokens {
                  asset {
                      assetId
                      assetName
                      decimals
                      description
                      fingerprint
                      logo
                      metadataHash
                      name
                      ticker
                      tokenMints {
                          quantity
                          transaction {
                              hash
                          }
                      }
                      tokenMints_aggregate {
                          aggregate {
                              avg {
                                  quantity
                              }
                              count
                              max {
                                  quantity
                              }
                              min {
                                  quantity
                              }
                              sum {
                                  quantity
                              }
                          }
                      }
                      url
                      policyId
                  }
                  quantity
              }
          }
          utxos_aggregate (
              where: {
                  _and: {
                      address: { _eq: $address },
                      transaction: { block: { number: { _lte: $atBlock }}}
                  }
              }
          ) {
              aggregate {
                  count
              }
          }
      }`, {
                address,
                atBlock
            });
            const map = new Map();
            for (const utxo of result.utxos) {
                if (map.has('ada')) {
                    const current = map.get('ada');
                    map.set('ada', Object.assign(Object.assign({}, current), {
                        quantity: new bignumber_js_1.default(current.quantity)
                            .plus(new bignumber_js_1.default(utxo.value))
                            .toString()
                    }));
                }
                else {
                    map.set('ada', {
                        asset: {
                            assetId: '\\xada',
                            assetName: '\\xada',
                            name: 'ada',
                            policyId: '\\xada',
                            tokenMints: [],
                            tokenMints_aggregate: {
                                aggregate: {
                                    avg: {
                                        quantity: 'na'
                                    },
                                    count: 'na',
                                    max: {
                                        quantity: 'na'
                                    },
                                    min: {
                                        quantity: 'na'
                                    },
                                    sum: {
                                        quantity: 'na'
                                    }
                                },
                                nodes: []
                            }
                        },
                        quantity: utxo.value
                    });
                }
                for (const token of utxo.tokens) {
                    if (map.has(token.asset.assetId)) {
                        const current = map.get(token.asset.assetId);
                        map.set(token.asset.assetId, Object.assign(Object.assign({}, current), {
                            quantity: new bignumber_js_1.default(current.quantity)
                                .plus(new bignumber_js_1.default(token.quantity))
                                .toString()
                        }));
                    }
                    else {
                        map.set(token.asset.assetId, token);
                    }
                }
            }
            return {
                assetBalances: [...map.values()],
                utxosCount: result.utxos_aggregate.aggregate.count
            };
        });
    }
    getMeta(nodeTipSlotNumber) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query {
          epochs (limit: 1, order_by: { number: desc }) {
              number
          }
          cardano {
              tip {
                  epoch {
                      number
                  }
                  slotNo
                  forgedAt
              }
          }}`);
            const { tip } = result === null || result === void 0 ? void 0 : result.cardano[0];
            const lastEpoch = result === null || result === void 0 ? void 0 : result.epochs[0];
            const syncPercentage = tip.slotNo / nodeTipSlotNumber * 100;
            return {
                initialized: lastEpoch.number === ((_a = tip.epoch) === null || _a === void 0 ? void 0 : _a.number),
                syncPercentage: syncPercentage > 100 ? 100 : syncPercentage
            };
        });
    }
    hasAsset(assetId) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query HasAsset (
          $assetId: bytea!
      ) {
          assets (
              where: { assetId: { _eq: $assetId }}
          ) {
              assetId
          }
      }`, {
                assetId: withHexPrefix(assetId)
            });
            const response = result.assets.length > 0;
            this.logger.debug({ module: 'HasuraClient', assetId, hasAsset: response }, 'Has asset?');
            return response;
        });
    }
    getAssetMetadataHashesById(assetIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.request(graphql_request_1.gql `query AssetMetadataHashes (
          $assetIds: [bytea!]!
      ){
          assets (
              where: {
                  assetId: { _in: $assetIds }
              }) {
              assetId
              metadataHash
          }
      }`, {
                assetIds: assetIds.map(id => withHexPrefix(id))
            });
            return result.assets;
        });
    }
    addAssetMetadata(asset) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info({ module: 'HasuraClient', assetId: asset.assetId }, 'Adding metadata to asset');
            const result = yield this.client.request(graphql_request_1.gql `mutation AddAssetMetadata(
          $assetId: bytea!
          $decimals: Int
          $description: String
          $logo: String
          $metadataHash: bpchar!
          $name: String
          $ticker: String
          $url: String
      ) {
          update_assets(
              where: {
                  assetId: { _eq: $assetId }
              },
              _set: {
                  decimals: $decimals
                  description: $description
                  logo: $logo
                  metadataHash: $metadataHash
                  name: $name
                  ticker: $ticker
                  url: $url
              }
          ) {
              affected_rows
              returning {
                  assetId
              }
          }
      }`, Object.assign(Object.assign({}, asset), { assetId: withHexPrefix(asset.assetId) }));
            if (result.errors !== undefined) {
                throw new Error(result.errors);
            }
        });
    }
    insertAssets(assets) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug({ module: 'HasuraClient', qty: assets.length }, 'inserting assets found in tokens');
            const result = yield this.client.request(graphql_request_1.gql `mutation InsertAssets($assets: [Asset_insert_input!]!) {
          insert_assets(objects: $assets) {
              returning {
                  name
                  policyId
                  description
                  assetName
                  assetId
              }
              affected_rows
          }
      }`, {
                assets: assets.map(asset => (Object.assign(Object.assign({}, asset), {
                    assetId: withHexPrefix(asset.assetId),
                    assetName: withHexPrefix(asset.assetName),
                    policyId: withHexPrefix(asset.policyId)
                })))
            });
            return result;
        });
    }
}
exports.HasuraClient = HasuraClient;
//# sourceMappingURL=HasuraClient.js.map