"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetadataClient = void 0;
const axios_1 = __importDefault(require("axios"));
const util_1 = require("@cardano-graphql/util");
const ts_log_1 = require("ts-log");
const MODULE_NAME = 'MetadataFetchClient';
class MetadataClient {
    constructor(metadataServerUri, logger = ts_log_1.dummyLogger) {
        this.metadataServerUri = metadataServerUri;
        this.logger = logger;
        this.state = null;
        this.axiosClient = axios_1.default.create({
            baseURL: this.metadataServerUri
        });
    }
    ensureMetadataServerIsAvailable() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.axiosClient.get('/metadata/healthcheck');
            }
            catch (error) {
                if (error.code === 'ENOTFOUND') {
                    throw new util_1.errors.HostDoesNotExist('metadata server');
                }
                else if (error.response.status !== 404) {
                    throw error;
                }
            }
        });
    }
    fetch(assetIds) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'initialized') {
                throw new util_1.errors.ModuleIsNotInitialized(MODULE_NAME, 'fetch');
            }
            try {
                const response = yield this.axiosClient.post('metadata/query', {
                    subjects: assetIds,
                    properties: [
                        'decimals',
                        'description',
                        'logo',
                        'name',
                        'ticker',
                        'url'
                    ]
                });
                return response.data.subjects;
            }
            catch (error) {
                if (error.code === 'ENOTFOUND') {
                    this.logger.error({ err: error });
                }
                else {
                    throw error;
                }
            }
        });
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== null)
                return;
            this.state = 'initializing';
            this.logger.info({ module: MODULE_NAME }, 'Initializing');
            yield this.ensureMetadataServerIsAvailable();
            this.state = 'initialized';
            this.logger.info({ module: MODULE_NAME }, 'Initialized');
        });
    }
}
exports.MetadataClient = MetadataClient;
//# sourceMappingURL=MetadataClient.js.map